/*
   http://glprogramming.com/red/chapter05.html
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.util.gl2.GLUT;

import javax.swing.JFrame;

/**

*/
public class Light implements GLEventListener
{
   public Light()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(500, 500));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Light");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);
   }

   // Implement the GLEventListener interface (four methods).
   @Override
   public void init(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      float[] mat_specular = {1.0f, 1.0f, 1.0f, 1.0f};
      float[] mat_shininess = {50.0f};
      float[] light_position = {1.0f, 1.0f, 1.0f, 0.0f};
      gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
      gl.glShadeModel(gl.GL_SMOOTH); // try GL_FLAT

      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, mat_shininess, 0);
      gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, light_position, 0);

      gl.glEnable(gl.GL_LIGHTING);
      gl.glEnable(gl.GL_LIGHT0);
      gl.glEnable(gl.GL_DEPTH_TEST);
   }


   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
      new GLUT().glutSolidSphere(1.0, 20, 16);
      gl.glFlush();
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(gl.GL_PROJECTION);
      gl.glLoadIdentity();
      if (w <= h)
         gl.glOrtho(-1.5, 1.5, -1.5*(float)h/(float)w,
                     1.5*(float)h/(float)w, -10.0, 10.0);
      else
         gl.glOrtho(-1.5*(float)w/(float)h,
                     1.5*(float)w/(float)h, -1.5, 1.5, -10.0, 10.0);
      gl.glMatrixMode(gl.GL_MODELVIEW);
      gl.glLoadIdentity();
   }


   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }


   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new Light()
      );
   }
}
