/*
   http://glprogramming.com/red/chapter05.html

   http://www.glprogramming.com/red/appendixi.html#plate16
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.util.gl2.GLUT;

import javax.swing.JFrame;

/**

*/
public class Light_v3 implements GLEventListener
{
   public Light_v3()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(1500, 500));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Light");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);
   }

   // Implement the GLEventListener interface (four methods).
   @Override
   public void init(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      float[] mat_specular = {1.0f, 1.0f, 1.0f, 1.0f};
      float[] mat_shininess = {50.0f};
      float[] light_position = {1.0f, 1.0f, 1.0f, 0.0f};
      gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
      gl.glShadeModel(gl.GL_SMOOTH); // try GL_FLAT

      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, mat_shininess, 0);
      gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, light_position, 0);

      gl.glEnable(gl.GL_LIGHTING);
      gl.glEnable(gl.GL_LIGHT0);
      gl.glEnable(gl.GL_DEPTH_TEST);
   }

   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      float[] no_mat = {0.0f, 0.0f, 0.0f, 1.0f};
      float[] mat_ambient = {0.7f, 0.7f, 0.7f, 1.0f};
      float[] mat_ambient_color = {0.8f, 0.8f, 0.2f, 1.0f};
      float[] mat_diffuse = {0.1f, 0.5f, 0.8f, 1.0f};
      float[] mat_specular = {1.0f, 1.0f, 1.0f, 1.0f};
      float[] no_shininess = {0.0f};
      float[] low_shininess = {5.0f};
      float[] high_shininess = {100.0f};
      float[] mat_emission = {0.3f, 0.2f, 0.2f, 0.0f};

      gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);

      // draw sphere in first row, first column
      // diffuse reflection only; no ambient or specular
      gl.glPushMatrix();
      gl.glTranslatef(-3.75f, 0.0f, 0.0f);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, mat_diffuse, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, no_shininess, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_EMISSION, no_mat, 0);
      new GLUT().glutSolidSphere(1.0, 16, 16);
      gl.glPopMatrix();

      // draw sphere in first row, second column
      // diffuse and specular reflection; low shininess; no ambient
      gl.glPushMatrix();
      gl.glTranslatef(-1.25f, 0.0f, 0.0f);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, mat_diffuse, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, low_shininess, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_EMISSION, no_mat, 0);
      new GLUT().glutSolidSphere(1.0, 16, 16);
      gl.glPopMatrix();

      // draw sphere in first row, third column
      // diffuse and specular reflection; high shininess; no ambient
      gl.glPushMatrix();
      gl.glTranslatef(1.25f, 0.0f, 0.0f);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, mat_diffuse, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, high_shininess, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_EMISSION, no_mat, 0);
      new GLUT().glutSolidSphere(1.0, 16, 16);
      gl.glPopMatrix();

      // draw sphere in first row, fourth column
      // diffuse reflection; emission; no ambient or specular refl.
      gl.glPushMatrix();
      gl.glTranslatef(3.75f, 0.0f, 0.0f);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, mat_diffuse, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, no_mat, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, no_shininess, 0);
      gl.glMaterialfv(gl.GL_FRONT, gl.GL_EMISSION, mat_emission, 0);
      new GLUT().glutSolidSphere(1.0, 16, 16);
      gl.glPopMatrix();
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(gl.GL_PROJECTION);
      gl.glLoadIdentity();
      if (w <= h)
         gl.glOrtho(-1.5, 1.5, -1.5*(float)h/(float)w,
                     1.5*(float)h/(float)w, -10.0, 10.0);
      else
         gl.glOrtho(-1.5*(float)w/(float)h,
                     1.5*(float)w/(float)h, -1.5, 1.5, -10.0, 10.0);
      gl.glMatrixMode(gl.GL_MODELVIEW);
      gl.glLoadIdentity();
   }


   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }


   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new Light_v3()
      );
   }
}
